package com.persagy.demo.netty.websocket;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.date.TimeInterval;
import lombok.extern.slf4j.Slf4j;
import org.java_websocket.WebSocket;
import org.java_websocket.client.WebSocketClient;
import org.java_websocket.handshake.ServerHandshake;

import java.net.URI;
import java.net.URISyntaxException;

/**
 * @description: TODO
 * @author:LuoGuangyi
 * @company:PersagyTechnologyCo.,Ltd
 * @since:2021/03/31 16:04
 * @version:V1.0
 **/
@Slf4j
public class WebSocketClientTest {
    private static WebSocketClient webSocketClient;

    public static void main(String[] args) throws URISyntaxException {
        createNewWebSocketClient();
    }

    /**
     * 创建websocket对象
     *
     * @return WebSocketClient
     * @throws URISyntaxException
     */
    private static WebSocketClient createNewWebSocketClient() throws URISyntaxException {
        String outCallWebSockertUrl = "ws://localhost:8888/websocket";
        log.info("连接地址为:[{}]", outCallWebSockertUrl);
        webSocketClient = new WebSocketClient(new URI(outCallWebSockertUrl)) {
            @Override
            public void onOpen(ServerHandshake serverHandshake) {
                String msg = "{\"type\":2,\"msgInfo\":\"11\"}";
                sendMsg(WebSocketClientTest.webSocketClient, msg);
            }

            @Override
            public void onMessage(String msg) {
                String message = msg;
                System.out.println("-----------------" + message);
            }

            @Override
            public void onClose(int code, String reason, boolean remote) {
                log.warn("关闭连接,code[{}],reson[{}],remote[{}]", code, reason, remote);
            }

            @Override
            public void onError(Exception e) {
                log.error("连接异常", e);
            }
        };
        //默认为60 ，阻塞时间稍长会超时，此处改成 1200
        webSocketClient.setConnectionLostTimeout(1200);
        webSocketClient.connect();
        return webSocketClient;
    }


    public static void sendMsg(WebSocketClient webSocketClient, String message) {
        log.info("websocket向服务端发送消息，消息为：{}", message);
        TimeInterval timer = DateUtil.timer();
        while (!webSocketClient.getReadyState().equals(WebSocket.READYSTATE.OPEN)) {
            log.debug("正在建立通道，请稍等");
            if (timer.interval() >= 5000) {
                log.error("超过5秒钟还未打开连接，超时，不再等待");
                return;
            }
        }
        webSocketClient.send(message);
    }
}