package com.persagy.entity;

import com.alibaba.fastjson.JSONObject;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.time.LocalDateTime;
import java.util.List;

/**
 * @description:报警定义条件
 * @author:LuoGuangyi
 * @company:PersagyTechnologyCo.,Ltd
 * @since:2020/10/20 002018:20
 * @version:V1.0 实例：
 * {
 * "infoCode": [
 * "infoCode1",
 * "infoCode1"
 * ],
 * "infoCodes": [
 * {
 * "infoCode": "infoCode1",
 * "meterId": "11",
 * "funcId": "12"
 * },
 * {
 * "infoCode": "infoCode2",
 * "meterId": "21",
 * "funcId": "22"
 * }
 * ],
 * "trigger": "(infoCode1 > 10 && infoCode1 < 20) && (infoCode2 < 20)",
 * "end": "!(infoCode1 < 20 && infoCode1 > 10)",
 * "$triggerUphold": 10,
 * "$endUphold": 5,
 * "$time": {
 * "type": "period",
 * "period": {
 * "$gt": "090000",
 * "$lt": "210000"
 * }
 * }
 * }
 **/
@Data
public class Condition {
    List<String> infoCode;
    List<JSONObject> infoCodes;
    private JSONObject configs;
    private String trigger;
    private String end;
    private int triggerUphold;
    private int endUphold;
    private JSONObject effectTime;
    /**
     * 抑制时间以分钟为单位
     */
    @ApiModelProperty(value = "抑制时间以分钟为单位")
    private Long restraintTime;

    @ApiModelProperty(value = "抑制时间,误报操作时间+抑制时间")
    private LocalDateTime realRestraintTime;

    @Override
    public String toString() {
        return JSONObject.toJSONString(this);
    }
}
