package com.persagy.init;


import cn.hutool.core.thread.ThreadUtil;
import com.googlecode.aviator.AviatorEvaluator;
import com.persagy.client.WebSocketClientFactory;
import com.persagy.constant.CommonConst;
import com.persagy.job.AlarmMessageThread;
import com.persagy.netty.client.NettyClient;
import com.persagy.service.AlarmQuartzService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.CommandLineRunner;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Service;

import javax.servlet.ServletContextAttributeListener;


@Service
@Order(1)
@Slf4j
public class InitRunner implements ServletContextAttributeListener, CommandLineRunner {

    @Autowired
    AlarmQuartzService alarmQuartzService;
    @Autowired
    private WebSocketClientFactory webSocketClientFactory;
    @Autowired
    private NettyClient nettyClient;
    @Value("${spring.kafka.enable:false}")
    private Boolean kafkaEnable;

    @Override
    public void run(String... args) throws Exception {

        //5.0 开始引入了 LRU 缓存，可指定缓存的表达式个数，比如设置为最大 1 万个缓存结果：
        AviatorEvaluator.getInstance().useLRUExpressionCache(10000);
        if(!kafkaEnable) {
            //启动netty客户端，接受云端数据
            nettyClient.connect();
        }
        //启动websocket,接受IOT采集庶数据
        webSocketClientFactory.retryOutCallWebSocketClient();
        //异步消费消息
        ThreadUtil.execAsync(new AlarmMessageThread(alarmQuartzService), true);
    }

    /**
     * 项目名称
     */
    @Value("${project.id}")
    public void setProjectId(String value) {
        CommonConst.projectId = value;
    }

    /**
     * 集团编码
     */
    @Value("${group.code}")
    public void setGroupCode(String value) {
        CommonConst.groupCode = value;
    }


    @Value("${group.alarm.host:127.0.0.1}")
    public void setInetHost(String value) {
        CommonConst.inetHost = value;
    }


    @Value("${group.alarm.port}")
    public void setInetPort(int value) {
        CommonConst.inetPort = value;
    }

    @Value("${system.id}")
    public void setSystemId(String value) {
        CommonConst.systemId = value;
    }

}
