package com.persagy.job;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.date.TimeInterval;
import com.alibaba.fastjson.JSONObject;
import com.persagy.constant.CommonConst;
import com.persagy.entity.NettyMessage;
import com.persagy.netty.client.NettyClient;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.Objects;

/**
 * @author LuoGuangyi
 * @title: SpringSchedule
 * @projectName spring-boot-examples
 * @description: TODO
 * @date 2019/09/17 16:54
 */

@Service
@Slf4j
public class SpringSchedule {
    @Autowired
    NettyClient nettyClient;
    @Value("${spring.kafka.enable:false}")
    private Boolean kafkaEnable;

    @Scheduled(cron = "${alarm.get.all.alarmdefine.cron}")
    public void allResetCron() throws InterruptedException {
        NettyMessage message;
        if(kafkaEnable){
            message = new NettyMessage(4,null);
        }else{
            message = new NettyMessage(4);
        }
        JSONObject content = new JSONObject();
        content.put("groupCode", CommonConst.groupCode);
        content.put("projectId", CommonConst.projectId);
        message.setContent(Arrays.asList(content));
        nettyClient.sendMessage(message);
    }

    @Scheduled(initialDelay = 2000, fixedDelay = 60000)
    public void connectCron() {
        if (Objects.isNull(NettyClient.channel)) {
            log.warn("NettyClient is not init");
            return;
        }
        NettyMessage message = new NettyMessage(3);
        try {
        nettyClient.sendMessage(message);
        } catch (Exception e) {
            log.error("定时任务发送报警消息失败", e);
        }
    }

    @Scheduled(initialDelay = 1000, fixedDelay = 600000)
    public void connectAnalizeCron() {
        if(kafkaEnable){
            return;
        }
        if (Objects.isNull(NettyClient.channel)) {
            log.warn("NettyClient is not init");
            return;
        }
        log.info("NettyClient State:isActive[{}],isOpen[{}],isRegistered[{}],isWritable[{}]", NettyClient.channel.isActive(), NettyClient.channel.isOpen(), NettyClient.channel.isRegistered(), NettyClient.channel.isWritable());
        sengAlarmMessage();
    }

    private void sengAlarmMessage() {
        try {
            log.info("--sengAlarmMessage--");
            TimeInterval timer = DateUtil.timer();
            while (NettyMessageQueue.getNettyMessageQueue().size() > 0 && timer.interval() < 100000) {
                NettyMessage msg = NettyMessageQueue.getNettyMessageQueue().consume();
                log.info("剩余报警消息令总数:{}", NettyMessageQueue.getNettyMessageQueue().size());
                nettyClient.sendMessage(msg);
            }
        } catch (Exception e) {
            log.error("发送报警消息失败", e);
        }
    }

}
