package com.persagy.netty.client;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.persagy.constant.CommonConst;
import com.persagy.entity.NettyMessage;
import com.persagy.job.NettyMessageQueue;
import com.persagy.kafka.KafkaProducer;
import com.persagy.repository.AlarmRecordRepository;
import com.persagy.service.AlarmDefineService;
import io.netty.bootstrap.Bootstrap;
import io.netty.channel.*;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.nio.NioSocketChannel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

@Slf4j
@Component
public class NettyClient {

    public static Channel channel;
    public Bootstrap bootstrap = new Bootstrap();
    @Autowired
    AlarmDefineService alarmDefineService;
    @Autowired
    AlarmRecordRepository alarmRecordRepository;
    @Autowired(required = false)
    private KafkaProducer kafkaProducer;

    @Value("${spring.kafka.enable:false}")
    private Boolean kafkaEnable;

    public void connect() {
        EventLoopGroup workerGroup = new NioEventLoopGroup();
        try {
            bootstrap.group(workerGroup);
            bootstrap.channel(NioSocketChannel.class);
            bootstrap.option(ChannelOption.AUTO_READ, true);
            bootstrap.handler(new CenterChannelInitializer(alarmDefineService, this, alarmRecordRepository));
            ChannelFuture f = bootstrap.connect(CommonConst.inetHost, CommonConst.inetPort).sync();
            channel = f.channel();
            // f.channel().closeFuture().sync();
        } catch (Exception e) {
            log.info("连接异常", e);
        } finally {
            //workerGroup.shutdownGracefully();
        }
    }


    public void reConnect() {
        // 加入断线后自动重连监听器
        channel = bootstrap.connect(CommonConst.inetHost, CommonConst.inetPort).addListener(new ChannelFutureListener() {
            @Override
            public void operationComplete(ChannelFuture future) throws Exception {
                if (future.cause() != null) {
                    log.info("Failed to connect: {}", future.cause());
                }
            }
        }).channel();
    }


    public void sendMessage(NettyMessage msg) throws InterruptedException {
        if(kafkaEnable){
            kafkaProducer.send(msg);
        }else {
            log.info("给云端发送数据：[{}]", msg);
            if (channel.isActive()&&channel.isWritable()) {
                try {
                    //!!!!!!注意，NettyMessage<T>是有泛型的，如果把发送一个数据，接收方必须有一个和<T>的全限定类名完全一样的实体类，
                    //发送方不应该限制接收方，所有使用JSONObject.parseObject(msg.toString(), NettyMessage.class)操作让泛型擦除掉
                    channel.writeAndFlush(JSONObject.parseObject(msg.toString(), new TypeReference<NettyMessage<JSONObject>>() {
                    }));
                } catch (Exception e) {
                    log.error("发送数据异常,放入缓冲队列中", e);
                    NettyMessageQueue.getNettyMessageQueue().produce(msg);
                    channel.close();
                }
            } else {
                log.warn("云端netty不可写，放入缓冲队列中[{}]", msg);
                NettyMessageQueue.getNettyMessageQueue().produce(msg);
            }
        }
    }

    public void sendMessageNotCheck(NettyMessage msg){
        try {
            channel.writeAndFlush(JSONObject.parseObject(msg.toString(), NettyMessage.class));
        } catch (Exception e) {
            log.error(e.getMessage(),e);
        }
    }

}
